"""added foreign key to user logged in

Peek Plugin Database Migration Script

Revision ID: 1176bc54abf3
Revises: 7e3f668edf0e
Create Date: 2021-03-22 10:35:44.330884

"""

# revision identifiers, used by Alembic.
revision = "1176bc54abf3"
down_revision = "7e3f668edf0e"
branch_labels = None
depends_on = None

from alembic import op


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_unique_constraint("uq_username", "UserLoggedIn", ["userName"])

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint("uq_username", "UserLoggedIn")
    # ### end Alembic commands ###
