
@import "@synerty/peek-plugin-base-js/src/styles/_variables";

:host {
    display: flex;
    flex-direction: column;
    height: 100%;
    width: 100%;
}

nz-card {
    width: 100%;
    
    ::ng-deep .ant-card-head {
        min-height: auto;
        background: $brand;
        
        .ant-card-head-title {
            padding: $general_padding_1 0;
            color: $white;
        }
    }
}

.card-content {
    display: flex;
    flex-direction: column;
    gap: $general_padding_2;
}

.table-container {
    width: 100%;
    overflow: auto;
}

.online-status {
    &.online {
        color: $success;
    }
    
    &.offline {
        color: $warning;
    }
    
    &.never {
        color: $danger;
    }
}

.action-cell {
    text-align: right;
}