h4 {
    margin-bottom: 16px;
    margin-left: 4px;
}

.vehicle-login-container {
    display: flex;
    flex-direction: column;
    width: 250px;
    align-items: center;
    margin: 0 auto;
}

.login-actions {
    display: flex;
    justify-content: center;
    width: 100%;
}

.vehicle-input {
    width: 100%;
}

.next-button {
    width: 100%;

    [nz-icon] {
        margin-left: 8px;
    }
}
